const { pool } = require("../config/database/database.config");

/*
Class responsible for making queries to the 
involvementLookup and involvementType database.
*/

/*
Function to add an involvement type to the database
*/
async function addInvolvementTypeIfNotExists(description) {
  console.log("adding involvement type: ", description);

  // Don't add involvement type if it exists
  const existingType = await getInvolvementType(description);
  if (existingType) {
    console.log("Involvement type already exists: ", description);
    return;
  }

  // Insert SQL query
  const query = `INSERT INTO involvementType (description) VALUES (?)`;

  // Attempt to insert involvement type into table
  try {
    const [results] = await pool.query(query, [description]);
    console.log(`Successfully added involvement type: ${description}`);
    return results.insertId;
  } catch (error) {
    console.error(`Error adding involvement type: ${description}`, error);
    throw error;
  }
}

async function getInvolvementType(description) {
  // SQL query to get a certain involvement type
  const query = "SELECT * FROM involvementType WHERE description = ?";

  try {
    console.log("Getting involvement type: ", description);
    const [rows] = await pool.query(query, [description]);
    if (rows.length > 0) {
      console.log("Found involvement type: ", description);
      return rows[0];
    } else {
      console.log("Involvement type not found, creating involvement now");

      // Create the involvement type
      const insertQuery =
        "INSERT INTO involvementType (description) VALUES (?)";
      const [result] = await pool.query(insertQuery, [description]);

      await addInvolvementIfNotExists(result.insertId);

      return result.insertId;
    }
  } catch (error) {
    console.error(`Error fetching involvement type ${description}:`, error);
    throw error;
  }
}

async function getInvolvementTypeById(involvementId) {
  // SQL query to get a certain involvement type
  const query = "SELECT * FROM involvementType WHERE involvementTypeId = ?";

  try {
    console.log("Getting involvement type with id: ", involvementId);
    const [rows] = await pool.query(query, [involvementId]);
    if (rows.length > 0) {
      console.log("Found involvement type: ", involvementId);
      return rows[0];
    } else {
      console.log("Involvement type not found: ", involvementId);
      return null;
    }
  } catch (error) {
    console.error(`Error fetching involvement type ${involvementId}:`, error);
    throw error;
  }
}

/*
Function to add an involvement to the involvementLookup table
*/
async function addInvolvementIfNotExists(involvementTypeId) {
  console.log("Adding involvement for type ID: ", involvementTypeId);

  // Don't add involvement if it exists
  const existingInvolvement = await getInvolvement(involvementTypeId);
  if (existingInvolvement) {
    console.log("Involvement already exists: ", involvementTypeId);
    return;
  }

  // Insert SQL query
  const query = `INSERT INTO involvementLookup (involvementType) VALUES (?)`;

  // Attempt to insert involvement into table
  try {
    await pool.query(query, [involvementTypeId]);
    console.log(
      `Successfully added involvement with type ID: ${involvementTypeId}`
    );
  } catch (error) {
    console.error(
      `Error adding involvement with type ID: ${involvementTypeId}`,
      error
    );
    throw error;
  }
}

async function getInvolvement(involvementType) {
  // SQL query to get a certain involvement
  const query = "SELECT * FROM involvementLookup WHERE involvementType = ?";

  try {
    console.log("Getting involvement: ", involvementType);
    const [rows] = await pool.query(query, [involvementType]);
    if (rows.length > 0) {
      console.log("Found involvement: ", involvementType);
      return rows[0];
    } else {
      console.log("Involvement not found: ", involvementType);
      return null;
    }
  } catch (error) {
    console.error(`Error fetching involvement ${involvementType}:`, error);
    throw error;
  }
}

async function checkInvolvementTypeTableExistence() {
  try {
    const [rows] = await pool.query("SHOW TABLES LIKE 'involvementType'");
    if (rows.length > 0) {
      console.log("Table `involvementType` exists");
    } else {
      console.log("Table `involvementType` does not exist");
    }
  } catch (error) {
    console.error("Error checking involvementType table existence:", error);
  }
}

async function checkInvolvementLookupTableExistence() {
  try {
    const [rows] = await pool.query("SHOW TABLES LIKE 'involvementLookup'");
    if (rows.length > 0) {
      console.log("Table `involvementLookup` exists");
    } else {
      console.log("Table `involvementLookup` does not exist");
    }
  } catch (error) {
    console.error("Error checking involvementLookup table existence:", error);
  }
}

async function addPeopleInvolvement(personId, involvementId) {
  const query = `
          INSERT INTO peopleXInvolvement (peopleId, involvementId)
          VALUES (?, ?)
          ON DUPLICATE KEY UPDATE peopleId = peopleId;
        `;

  try {
    const [results] = await pool.query(query, [personId, involvementId]);

    console.log(
      `Linked involvement ${involvementId} to person ID: ${personId}`
    );

    console.log("add involvement results: ", results);
    console.log("returning: ", results.insertId);
    return results.insertId;
  } catch (error) {
    console.log(
      `Error linking involvement ${involvementId} to person ID: ${personId}: `,
      error
    );
  }
}

/*
Function to delete an involvement for a person by their ID and involvement ID.
*/
async function deleteInvolvementForPerson(personId, involvementId) {
  const selectQuery = `
    SELECT * 
    FROM peopleXInvolvement
    WHERE peopleId = ? AND involvementId = ?
  `;

  const deleteQuery = `
    DELETE FROM peopleXInvolvement
    WHERE peopleId = ? AND involvementId = ?
  `;

  try {
    // Fetch the row to be deleted
    const [rows] = await pool.query(selectQuery, [personId, involvementId]);

    if (rows.length === 0) {
      console.log(
        `No involvement ${involvementId} found for person ID: ${personId}`
      );
      return null; // No row to delete
    }

    const deletedRow = rows[0]; // Get the row to be deleted

    // Perform the deletion
    const [result] = await pool.query(deleteQuery, [personId, involvementId]);

    if (result.affectedRows > 0) {
      console.log(
        `Successfully deleted involvement ${involvementId} for person ID: ${personId}`
      );
      return deletedRow; // Return the deleted row data
    } else {
      console.log(
        `Deletion failed for involvement ${involvementId} and person ID: ${personId}`
      );
      return null; // Indicate deletion failed
    }
  } catch (error) {
    console.error(
      `Error deleting involvement ${involvementId} for person ID: ${personId}:`,
      error
    );
    return null;
  }
}

/*
Function to get all involvement names associated with a person by their ID.
*/
async function getInvolvementsForPerson(personId) {
  const query = `
    SELECT it.description
    FROM peopleXInvolvement px
    JOIN involvementLookup il ON px.involvementId = il.involvementId
    JOIN involvementType it ON il.involvementType = it.involvementTypeId
    WHERE px.peopleId = ?
  `;

  try {
    const [rows] = await pool.query(query, [personId]);
    const involvementNames = rows.map((row) => row.description);
    console.log(`Involvements for person ID ${personId}:`, involvementNames);
    return involvementNames;
  } catch (error) {
    console.error(
      `Error fetching involvements for person ID ${personId}:`,
      error
    );
    throw error;
  }
}

module.exports = {
  checkInvolvementTypeTableExistence,
  checkInvolvementLookupTableExistence,
  addInvolvementTypeIfNotExists,
  getInvolvementType,
  addInvolvementIfNotExists,
  getInvolvement,
  addPeopleInvolvement,
  getInvolvementsForPerson,
  deleteInvolvementForPerson,
  getInvolvementTypeById,
};
