const { pool } = require("../config/database/database.config");
const {
  getChapterId,
  getChapterById,
} = require("../repository/chapterRepository");

async function addPerson(person) {
  // extract all the values from person
  let {
    firstName,
    lastName,
    middleName,
    maidenName,
    suffix,
    nickName,
    techAlumniChapter,
    classYear,
    gradYear,
    gradSemester,
    gender,
  } = person;

  // lookup the chapter id from the name
  techAlumniChapter = await getChapterId(techAlumniChapter);

  try {
    console.log("Adding person: ", firstName);

    const query = `
      INSERT INTO people (firstName, lastName, middleName, maidenName, suffix, nickName, techAlumniChapter, classYear, gradYear, gradSemester, gender)
      VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
    `;

    const [results] = await pool.query(query, [
      firstName,
      lastName,
      middleName,
      maidenName,
      suffix,
      nickName,
      techAlumniChapter,
      classYear,
      gradYear,
      gradSemester,
      gender,
    ]);

    // return the new person's id
    return results.insertId;
  } catch (error) {
    console.error(error);
    return null;
  }
}

/*
Function to search for people by their first and last name, 
if multiple people with that name exist, they will all be returned. 
If no people are found, the function will return null.
*/
async function searchPeopleByName(firstName, lastName) {
  console.log("searching for people by name....");

  try {
    // SQL query to search for people by first and last name
    const query = `
      SELECT * FROM people
      WHERE firstName = ? AND lastName = ?
      `;

    // Execute the query and pass the first and last name as parameters
    let [results] = await pool.query(query, [firstName, lastName]);

    console.log("results found: ", results);

    // If no people are found, return null
    if (results.length === 0) {
      return null;
    }

    // loop through the people found and add chapter info
    for (let result of results) {
      result.techAlumniChapter = await getChapterById(result.techAlumniChapter);
    }

    // return the people found
    console.log("results: ", results);
    return results;
  } catch (error) {
    console.error(error);
    return null;
  }
}

/*
Function to get the data associated with a specific peopleId.
If none is found, the function returns null.
*/
async function getPersonById(peopleId) {
  console.log("searching for person...");

  try {
    // big query to get all of the data
    const query = `
    SELECT 
      peopleId, 
      firstName, 
      lastName, 
      middleName, 
      maidenName, 
      suffix, 
      nickName, 
      techAlumniChapter, 
      classYear, 
      gradYear, 
      gradSemester, 
      gender
    FROM 
      people
    WHERE 
      peopleId = ?;
  `;

    // Execute the query
    const [rows] = await pool.query(query, [peopleId]);

    if (rows.length === 0) {
      console.log("No person found");
      null;
    }

    // structure the data
    console.log("data recieved... structuring now");

    // create the structure of a person
    let person = rows[0];

    // get the chapter name
    person.techAlumniChapter = await getChapterById(person.techAlumniChapter);

    return person;
  } catch (error) {
    console.log("Error getting person's full details: ", error);
    return null;
  }
}

/*
Function to delete a person with the peopleId given. 
Since the SQL schema has ON DELETE CASCADE, all associated data
(address, contacts, etc). will also be deleted. Function returns
true if the delete was successful and false if not.
*/
async function deletePersonById(peopleId) {
  console.log("Deleting person by ID...");

  try {
    // Fetch the person details before deletion
    const selectQuery = `SELECT * FROM people WHERE peopleId = ?`;
    const [personResults] = await pool.query(selectQuery, [peopleId]);

    if (personResults.length === 0) {
      console.log("No person found with the given ID.");
      return {
        success: false,
        message: "Person not found.",
        deletedData: null,
      };
    }

    const personToDelete = personResults[0]; // Capture the person's details

    // Delete the person
    const deleteQuery = `DELETE FROM people WHERE peopleId = ?`;
    const [deleteResults] = await pool.query(deleteQuery, [peopleId]);

    if (deleteResults.affectedRows === 0) {
      console.log("No rows deleted.");
      return {
        success: false,
        message: "Failed to delete person.",
        deletedData: null,
      };
    }

    console.log("Person deleted successfully.");

    // Return the deleted person's details
    return {
      success: true,
      message: "Person deleted successfully.",
      deletedData: personToDelete,
    };
  } catch (error) {
    console.error("Error deleting person:", error);
    return { success: false, message: "Error deleting person.", error };
  }
}

/*
Function to update the people table's information for
the person with the given id.
*/
async function updatePerson(personId, personInfo) {
  // extract all the values from person
  let {
    firstName,
    lastName,
    middleName,
    maidenName,
    suffix,
    nickName,
    techAlumniChapter,
    classYear,
    gradYear,
    gradSemester,
    gender,
  } = personInfo;

  // lookup the chapter id from the name
  const chapterId = await getChapterId(techAlumniChapter);

  try {
    console.log("Updating person: ", firstName);

    // SQL UPDATE query
    const query = `
     UPDATE people
     SET 
       firstName = ?, 
       lastName = ?, 
       middleName = ?, 
       maidenName = ?, 
       suffix = ?, 
       nickName = ?, 
       techAlumniChapter = ?, 
       classYear = ?, 
       gradYear = ?, 
       gradSemester = ?, 
       gender = ?
     WHERE peopleId = ?
   `;

    const [results] = await pool.query(query, [
      firstName,
      lastName,
      middleName,
      maidenName,
      suffix,
      nickName,
      chapterId,
      classYear,
      gradYear,
      gradSemester,
      gender,
      personId,
    ]);

    // Check if any rows were affected
    if (results.affectedRows === 0) {
      console.error("Person not found. no rows updated");
      return false;
    }

    console.log("person info updated: ", personId);

    // Fetch the updated row
    const selectQuery = `SELECT * FROM people WHERE peopleId = ?`;
    const [updatedPerson] = await pool.query(selectQuery, [personId]);

    if (updatedPerson.length === 0) {
      console.error("Failed to retrieve updated person.");
      return null;
    }

    return updatedPerson[0]; // Return the updated row
  } catch (error) {
    console.error(error);
    return false;
  }
}

// Define a whitelist of allowed fields to prevent SQL injection
const allowedFilters = [
  "firstName",
  "lastName",
  "nickName",
  "classYearStart",
  "classYearEnd",
  "gradYearStart",
  "gradYearEnd",
  "degreeYear",
  "degreeDepartment",
  "degreeCollege",
  "city",
  "stateId",
  "involvementType",
];

async function search(filters) {
  //begin by selecting all people and concatenating their fields with multiple values
  let baseQuery = `
    SELECT 
      p.peopleId,
      p.firstName,
      p.lastName,
      p.middleName,
      p.maidenName,
      p.suffix,
      p.nickName,
      p.techAlumniChapter,
      p.classYear,
      p.gradYear,
      p.gradSemester,
      p.gender,
      GROUP_CONCAT(DISTINCT CONCAT(pd.degreeYear, ' - ', pd.degreeDepartment, ' (', pd.degreeCollege, ')') SEPARATOR ' | ') AS degrees,
      GROUP_CONCAT(DISTINCT CONCAT(a.city, ', ', sl.name) SEPARATOR ' | ') AS addresses,
      GROUP_CONCAT(DISTINCT c.contactNumber SEPARATOR ', ') AS contactNumbers,
      GROUP_CONCAT(DISTINCT il.involvementType SEPARATOR ', ') AS involvements
    FROM people p
    LEFT JOIN peopleDegree pd ON p.peopleId = pd.peopleId
    LEFT JOIN peopleXAddress pa ON p.peopleId = pa.peopleId
    LEFT JOIN address a ON pa.addressId = a.addressId
    LEFT JOIN stateLookup sl ON a.stateId = sl.stateId
    LEFT JOIN peopleContact c ON p.peopleId = c.peopleId
    LEFT JOIN peopleXInvolvement pi ON p.peopleId = pi.peopleId
    LEFT JOIN involvementLookup il ON pi.involvementId = il.involvementId
    WHERE p.peopleId IN (
        SELECT DISTINCT p2.peopleId
        FROM people p2
        LEFT JOIN peopleDegree pd2 ON p2.peopleId = pd2.peopleId
        LEFT JOIN peopleXAddress pa2 ON p2.peopleId = pa2.peopleId
        LEFT JOIN address a2 ON pa2.addressId = a2.addressId
        LEFT JOIN peopleContact c2 ON p2.peopleId = c2.peopleId
        LEFT JOIN peopleXInvolvement pi2 ON p2.peopleId = pi2.peopleId
        LEFT JOIN involvementLookup il2 ON pi2.involvementId = il2.involvementId
        WHERE 1=1
    `;

  // Query values for parameterized query
  let queryValues = [];

  // Check each filter field and add to the query if not null
  if (filters.firstName) {
    baseQuery += ` AND p2.firstName LIKE ?`;
    queryValues.push(`%${filters.firstName}%`);
  }
  if (filters.lastName) {
    baseQuery += ` AND p2.lastName LIKE ?`;
    queryValues.push(`%${filters.lastName}%`);
  }
  if (filters.nickName) {
    baseQuery += ` AND p2.nickName LIKE ?`;
    queryValues.push(`%${filters.nickName}%`);
  }

  // Handle classYearStart and classYearEnd
  if (filters.classYearStart) {
    baseQuery += ` AND p2.classYear >= ?`;
    queryValues.push(filters.classYearStart);
  }
  if (filters.classYearEnd) {
    baseQuery += ` AND p2.classYear <= ?`;
    queryValues.push(filters.classYearEnd);
  }

  // Handle gradYearStart and gradYearEnd
  if (filters.gradYearStart) {
    baseQuery += ` AND p2.gradYear >= ?`;
    queryValues.push(filters.gradYearStart);
  }
  if (filters.gradYearEnd) {
    baseQuery += ` AND p2.gradYear <= ?`;
    queryValues.push(filters.gradYearEnd);
  }

  if (filters.degreeYearStart) {
    baseQuery += ` AND pd2.degreeYear >= ?`;
    queryValues.push(filters.degreeYearStart);
  }

  if (filters.degreeYearEnd) {
    baseQuery += ` AND pd2.degreeYear <= ?`;
    queryValues.push(filters.degreeYearEnd);
  }

  if (filters.degreeDepartment) {
    baseQuery += ` AND pd2.degreeDepartment LIKE ?`;
    queryValues.push(`%${filters.degreeDepartment}%`);
  }
  if (filters.degreeCollege) {
    baseQuery += ` AND pd2.degreeCollege LIKE ?`;
    queryValues.push(`%${filters.degreeCollege}%`);
  }
  if (filters.city) {
    baseQuery += ` AND a2.city LIKE ?`;
    queryValues.push(`%${filters.city}%`);
  }
  if (filters.stateId) {
    baseQuery += ` AND a2.stateId = ?`;
    queryValues.push(filters.stateId);
  }
  if (filters.involvementType && filters.involvementType.length > 0) {
    baseQuery += ` AND il2.involvementType IN (?)`;
    queryValues.push(filters.involvementType);
  }

  // Finalize the query
  baseQuery += `
  )
     GROUP BY 
       p.peopleId, p.firstName, p.lastName, p.middleName, p.maidenName, p.suffix, 
       p.nickName, p.techAlumniChapter, p.classYear, p.gradYear, p.gradSemester, 
       p.gender;
   `;

  try {
    const [results] = await pool.query(baseQuery, queryValues);
    return results;
  } catch (error) {
    console.log("Error in repository w/ base query");
    throw error;
  }
}

module.exports = {
  addPerson,
  searchPeopleByName,
  deletePersonById,
  updatePerson,
  getPersonById,
  search,
};
