const { check, body, validationResult } = require("express-validator");
const express = require("express");
const peopleRouter = express.Router();
const {
  createPerson,
  readFile,
  findByName,
  deleteById,
  updatePersonById,
  getPersonFullDetails,
  generalSearch,
} = require("../service/peopleService");
const multer = require("multer");
const {
  authenticateAdminAndPersonal,
  authenticateAdminOnly,
} = require("../config/auth/authenticateJWT");
const fs = require("fs");
const path = require("path");

const uploadDir = path.join(__dirname, "uploads"); // Absolute path to the uploads folder

if (!fs.existsSync(uploadDir)) {
  fs.mkdirSync(uploadDir, { recursive: true }); // Create the directory if it doesn't exist
}

// const upload = multer({ storage: storage });
const storage = multer.diskStorage({
  destination: (req, file, cb) => {
    const uploadDir = path.join(__dirname, "uploads"); // Ensure absolute path
    if (!fs.existsSync(uploadDir)) {
      fs.mkdirSync(uploadDir, { recursive: true }); // Create folder if not exists
    }
    cb(null, uploadDir); // Use the absolute path to save files
  },
  filename: (req, file, cb) => {
    cb(null, Date.now() + "-" + file.originalname); // Unique filename to avoid collisions
  },
});

const upload = multer({ storage: storage });

/*
Function to check that the POST request body in 
create person contains the correct data types 
*/
const validateCreatePerson = [
  // Validate personInfo
  check("personInfo.firstName")
    .notEmpty()
    .withMessage("First name is required")
    .isString()
    .withMessage("First name must be a string"),

  check("personInfo.lastName")
    .notEmpty()
    .withMessage("Last name is required")
    .isString()
    .withMessage("Last name must be a string"),

  check("personInfo.classYear")
    .notEmpty()
    .withMessage("Class year is required")
    .isInt()
    .withMessage("Class year must be an integer"),

  // check if greater than 0
  check("personInfo.gradYear")
    .optional()
    .isInt()
    .withMessage("Grad year must be an integer if provided"),

  check("personInfo.gradSemester")
    .optional()
    .isString()
    .withMessage("Grad semester must be an String if provided"),

  check("personInfo.gender")
    .isIn(["male", "female", "other"])
    .withMessage("Invalid gender"),

  check("personInfo.maidenName")
    .optional()
    .isString()
    .withMessage("Maiden name should be a string"),

  // Validate degrees array
  body("degrees").isArray().withMessage("Degrees should be an array"),

  body("degrees.*.degreeType").isString().withMessage("Invalid degree type"),

  body("degrees.*.degreeYear")
    .notEmpty()
    .withMessage("Degree year is required")
    .isInt()
    .withMessage("Degree year must be an integer"),

  body("degrees.*.degreeDescription")
    .optional()
    .isLength({ max: 255 })
    .withMessage("Degree description cannot exceed 255 characters"),

  // Validate addresses array
  body("addresses").isArray().withMessage("Addresses should be an array"),

  body("addresses.*.address1")
    .notEmpty()
    .withMessage("Address line 1 is required"),

  body("addresses.*.city").notEmpty().withMessage("City is required"),

  body("addresses.*.state").notEmpty().withMessage("State is required"),

  body("addresses.*.country").notEmpty().withMessage("Country is required"),

  body("addresses.*.zipCode")
    .optional()
    .isInt()
    .withMessage("Zip code must be an integer if provided"),

  // Validate contacts array
  body("contacts").isArray().withMessage("Contacts should be an array"),

  body("contacts.*.contactNumber")
    .notEmpty()
    .withMessage("Contact number is required")
    .isString()
    .withMessage("Contact number must be a string"),

  body("contacts.*.contactType")
    .isIn(["phone", "email"])
    .withMessage("Invalid contact type"),

  body("contacts.*.preferredContact")
    .isBoolean()
    .withMessage("Preferred contact must be a boolean"),

  // Validate involvements
  body("involvements")
    .optional()
    .isArray()
    .withMessage("Involvements should be an array"),

  body("involvements.*")
    .optional()
    .isString()
    .withMessage("Each involvement must be a string"),
];

peopleRouter.post(
  "/create",
  authenticateAdminOnly,
  validateCreatePerson,
  async (req, res) => {
    // ensure the request body has the correct data types
    const errors = validationResult(req);

    // error found: return early
    if (!errors.isEmpty()) {
      return res.status(400).json({ errors: errors.array() });
    }

    try {
      const userId = req.user.peopleId;

      const person = await createPerson(req.body, userId);
      console.log("In people routes after add: ", person);

      return res
        .status(201)
        .json({ message: "person added successfully", personId: person });
    } catch (error) {
      console.log(error);
      return res.status(500).json({ message: error });
    }
  }
);

// POST route to handle the file and create many people
peopleRouter.post("/create-many", upload.single("file"), async (req, res) => {
  // Ensure the request body has the correct data types
  // const errors = validationResult(req);
  // if (!errors.isEmpty()) {
  //   return res.status(400).json({ errors: errors.array() });
  // }
  console.log("in api call");

  if (!req.file) {
    console.log("no file");
    return res.status(400).json({ message: "No file uploaded" });
  }
  console.log("before try");
  try {
    console.log("A");
    // Process the uploaded file with your existing readFile function
    const filePath = req.file.path;
    console.log("B");

    await readFile(filePath); // Pass the file path to your readFile function
    console.log("C");

    return res.status(201).json({
      message: "People added successfully",
    });
  } catch (error) {
    console.log("ERRORAAAA");
    console.error("Error processing file:", error);
    return res.status(500).json({
      message: "An error occurred while processing the file",
      error: error.message,
    });
  }
});

/*
Route to search for people by first and last name. Request body takes
firstName and lastName as strings.
*/
peopleRouter.post("/searchByName", authenticateAdminOnly, async (req, res) => {
  // get the first and last name
  const { firstName, lastName } = req.body;

  // Basic validation to ensure firstName and lastName are provided
  if (!firstName || !lastName) {
    return res
      .status(400)
      .json({ message: "First name and last name are required." });
  }

  try {
    // run function to get person by name
    const people = await findByName(firstName, lastName);

    // no person found
    if (people == null) {
      return res.status(404).json({ message: "person not found" });
    }

    // return the found people
    return res.status(200).json({ people: people });
  } catch (error) {
    // error
    console.log("error");
    return res.status(500).json({ message: error });
  }
});

// DELETE endpoint to delete a person by their ID
peopleRouter.delete("/:id", authenticateAdminOnly, async (req, res) => {
  const personId = parseInt(req.params.id, 10); // Convert to an integer

  // Check if personId is a valid integer
  if (!Number.isInteger(personId)) {
    return res
      .status(400)
      .json({ message: "Invalid ID. ID must be an integer." });
  }

  try {
    const userId = req.user.peopleId;

    // Call the function to delete the person
    const success = await deleteById(personId, userId);

    if (!success) {
      // If no person is deleted, return a 404 status
      return res
        .status(404)
        .json({ message: "Person not found or could not be deleted." });
    }

    // TODO: log database change here

    // Return a success message
    res.json({ message: "Person successfully deleted." });
  } catch (error) {
    console.error("Error in DELETE /person/:id", error);
    res.status(500).json({ message: "Internal server error." });
  }
});

peopleRouter.put(
  "/update/:id",
  authenticateAdminAndPersonal,
  async (req, res) => {
    // get the personId from the URL params
    const personId = parseInt(req.params.id, 10);

    // Check if personId is a valid integer
    if (!Number.isInteger(personId)) {
      return res
        .status(400)
        .json({ message: "Invalid ID. ID must be an integer." });
    }

    try {
      const userId = req.user.peopleId;

      // Call the update service
      console.log("req.body: ", req.body);
      const success = await updatePersonById(personId, req.body, userId);

      // no person was updated
      if (!success) {
        return res.status(404).json({ message: "Person not found" });
      }

      return res
        .status(200)
        .json({ message: "Person updated successfully", personId });
    } catch (error) {
      console.error("Error in PUT /update/:id", error);
      return res.status(500).json({ message: "Internal server error" });
    }
  }
);

peopleRouter.get(
  "/fullInfo/:id",
  authenticateAdminAndPersonal,
  async (req, res) => {
    // get the personId from the URL params
    const personId = parseInt(req.params.id, 10);

    // Check if personId is a valid integer
    if (!Number.isInteger(personId)) {
      return res
        .status(400)
        .json({ message: "Invalid ID. ID must be an integer." });
    }

    try {
      const person = await getPersonFullDetails(personId);

      if (person == null) {
        return res.status(404).json({ message: "Person not found" });
      }

      return res.status(200).json({ message: "Person found", person });
    } catch (error) {
      console.error("Error in GET /fullInfo/:id", error);
      return res.status(500).json({ message: "Internal server error" });
    }
  }
);

/*
No token auth needed for this, anyone can hit this endpoint.
*/
peopleRouter.post("/search", async (req, res) => {
  // filters is an array of { field, value } pairs
  let filters = req.body.filters;

  if (!filters) {
    return res.status(400).json({ message: "Filters not provided!" });
  }

  try {
    // get the list of people who match the given filters
    const results = await generalSearch(filters);
    res.status(200).json({ people: results });
  } catch (error) {
    console.log("Error in search route: ", error);
    res.status(500).json({ message: `Internal service error: ${error}` });
  }
});

module.exports = peopleRouter;
