import React, { useState } from "react";
import axios from "axios";
import styles from "./ApplicationActivityStyles.module.css";

const ApplicationActivity = () => {
  const [activityType, setActivityType] = useState("dbChanges"); // dbChanges or loginData
  const [startDate, setStartDate] = useState("");
  const [endDate, setEndDate] = useState("");
  const [activityData, setActivityData] = useState([]);
  const [loading, setLoading] = useState(false);
  const [error, setError] = useState(null);

  const handleFetchActivity = async () => {
    setLoading(true);
    setError(null);
    setActivityData([]);

    if (!startDate || !endDate) {
      setError("Please select a valid date range.");
      setLoading(false);
      return;
    }

    const token = localStorage.getItem("jwtToken");

    if (!token) {
      console.log("No token found");
      return;
    }

    const endpoint =
      activityType === "dbChanges"
        ? `/api/activity/db-changes/`
        : `/api/activity/login-data/`;

    const url = `http://localhost:3000${endpoint}?start=${startDate}&end=${endDate}`;

    try {
      const response = await axios.get(url, {
        headers: {
          Authorization: `Bearer ${token}`,
        },
      });
      setActivityData(response.data || []);
    } catch (err) {
      setError(
        err.response?.data?.message || "An error occurred while fetching data."
      );
    } finally {
      setLoading(false);
    }
  };

  const renderTableRows = (data) => {
    return data.map((row, index) => (
      <tr key={index}>
        {Object.entries(row).map(([key, value], idx) => (
          <td key={idx}>
            {/* Handle nested objects/arrays */}
            {typeof value === "object" && value !== null
              ? JSON.stringify(value)
              : value}
          </td>
        ))}
      </tr>
    ));
  };

  const renderTableHeaders = (data) => {
    if (data.length === 0) return null;
    return (
      <tr>
        {Object.keys(data[0]).map((key) => (
          <th key={key}>{key}</th>
        ))}
      </tr>
    );
  };

  return (
    <div className={styles.container}>
      <h2>User Activity</h2>
      <div className={styles.form}>
        {/* Activity Type Dropdown */}
        <label className={styles.inputField}>
          Activity Type:
          <select
            value={activityType}
            onChange={(e) => setActivityType(e.target.value)}
          >
            <option value="dbChanges">Database Changes</option>
            <option value="loginData">Login Activity</option>
          </select>
        </label>

        {/* Date Range Inputs */}
        <label className={styles.inputField}>
          Start Date:
          <input
            type="date"
            value={startDate}
            onChange={(e) => setStartDate(e.target.value)}
          />
        </label>

        <label className={styles.inputField}>
          End Date:
          <input
            type="date"
            value={endDate}
            onChange={(e) => setEndDate(e.target.value)}
          />
        </label>

        {/* Fetch Activity Button */}
        <button
          className={styles.fetchButton}
          onClick={handleFetchActivity}
          disabled={loading}
        >
          {loading ? "Loading..." : "Fetch Activity"}
        </button>
      </div>

      {/* Error Message */}
      {error && <p className={styles.error}>{error}</p>}

      {/* Activity Data Table */}
      {activityData.length > 0 && (
        <div className={styles.results}>
          <h3>Activity Results</h3>
          <table>
            <thead>{renderTableHeaders(activityData)}</thead>
            <tbody>{renderTableRows(activityData)}</tbody>
          </table>
        </div>
      )}
    </div>
  );
};

export default ApplicationActivity;
